/*
** PCX ROUTINES
**
** These routines are adapted from the Metagraphics MetaWINDOW version 3.7b
** PCX read functions.  They have been adapted mainly to read and return a
** single scan line at a time.
*/

#include    <stdlib.h>
#include    <stdio.h>
#include    "pcxstuff.h"


/***************************************************************************
**
** Open_PCX
**
** Read a file and check that it is a valid PCX file. Also check that the
** type of image in the PCX file is compatible with the current port
*/
int Open_PCX( char *fname , ImageDataType *img)
{
    FILE            *datfile;

    if( ( datfile = fopen( fname,"r+b") ) == NULL ) /* binary mode read */
        return 1;

    /*
    ** Read the PCX header
    */
    if( ( fread( &img->PCXInfo,sizeof(img->PCXInfo),1,datfile) ) == 0 ) {
        fclose( datfile );
        return 1;
    }

    /*
    ** Validate that the type is understandable
    */
    if( img->PCXInfo.dMAN != 10 )
        return 1;

    /*
    ** Make sure we understand the encoding scheme
    */
    if( img->PCXInfo.encoding != 1 )
        return 1;

    /*
    ** Restrict to 8 bits per pixel file types
    */
    if( img->PCXInfo.pixbits != 8) {
        return 1;
    }

    /*
    ** Build an image header for this guy
    */
    img->f          = datfile;
    img->imgstart   = ftell(datfile);
/*    img->width      = img->PCXInfo.maxX - img->PCXInfo.minX + 1; */
    img->width      = img->PCXInfo.picBytes;
    img->depth      = img->PCXInfo.maxY - img->PCXInfo.minY + 1;
    PCXPalette(img);
    /*
    ** Get the file size
    */
    fseek(datfile, 0l, SEEK_END);
    img->size = ftell(datfile);

    return 0;
}



/*****************************************************************************
**
** PCX_Get_Row
**
** Get one scan line from the PCX image
**
*/
int PCX_Get_Row(FILE *datfile, PCXFileHDR *PCXInfo, int rows, unsigned char *line, int init)
{
    register unsigned char unwork, unwork2;
    register int    x, i;
    word            z;
    static int      lineCnt;

    /*
    ** If initializing then position file pointer to beginning of image data
    */
    if (init) {
        fseek(datfile, 0, SEEK_SET);
        fread( PCXInfo, sizeof(PCXFileHDR), 1, datfile );
        lineCnt = rows;
        return 1;
    }

    for( x = 0;  x < rows;   ) {
/*    for( x = 0;  x < PCXInfo->picBytes; ) { */

        unwork= (unsigned char) getc(datfile);
/*        if (x < rows) { */
        if( (unwork & PACKED) == PACKED ) {
            z = unwork & MASK;          /* calculate byte value             */
            unwork2 = (unsigned char) getc(datfile);
            for( i = 0; i < z; i++ ) {  /* unpack info to buffer            */
                line[x++] = unwork2;
            }
        }
        else {                         /* not packed                       */
            line[x++] = unwork;
        }
/*
        }
        else {
            x++;
        }
*/
    }
    return OK;
}


/***************************************************************************
**
** PCXPalette
**
** Load palette information from PCX file
*/
void PCXPalette(ImageDataType *img)
{
    int         i,j,k,t;
    long  int   fpos;
    PCXFileHDR  PCXInfo;

    /*
    ** Set to the beginning of the file and read the header again
    */
    fseek(img->f, 0, SEEK_SET);
    if( fread( &PCXInfo, sizeof(PCXInfo), 1, img->f) == 0 ) {
       return ;
    }

    /*
    ** First assume the image has only the 16 header colors...
    */
    for( i = 0; i < 16; i++ ) {
        j= i*3;
        img->palette[j] = PCXInfo.clrmap[j];
        img->palette[j+1] = PCXInfo.clrmap[j+1];
        img->palette[j+2] = PCXInfo.clrmap[j+2];
    }

    /*
    ** Seek to 769 bytes from end of file
    */
    fseek( img->f, 0L, SEEK_END );
    fpos = ftell( img->f);
    fseek( img->f, fpos - 769L, SEEK_SET );

    /*
    ** If there's a 12 there then this image has extended palette
    */
    i = getc(img->f);
    if( i != 12 ) {
        img->ncolors = 16;
        return;
    }
    img->ncolors = 256;
   
    /*
    ** Read the PCX palette
    */
    for(i=0;i<256;i++ ) {
        j = i*3;
        img->palette[j] = getc(img->f);
        img->palette[j+1] = getc(img->f);
        img->palette[j+2] = getc(img->f);
    }
    /*
    ** If there are more colors in the image than the display supports, we
    ** can sometimes get a better mapping if we randomize the image colors,
    ** since the TWS color system will load the first 8 colors from the
    ** image palette, then map the rest of the colors to those 8.
    ** Ask the user if he wants to randomize the colors.
    */
    if (img->ncolors > SM_nSystemColors()) {
        /*
        ** Use one of the built-in standard dialogs to query the user
        */
        j = SM_YesNoDialog("Attention: The image contains more colors than the graphics hardware can support.  Sometimes display results are better if image colors are first randomized.  Shall I do this?", "Yes", "No");
        if (!j) {                       /* NOTE: *LEFT* button returns 0!   */
            /*
            ** Randomly swap array elements into the first n positions
            */
            for (i = 0; i < SM_nApplicationColors(); i++) {
                j = i*3;
                k = random(256)*3;
                t = img->palette[j];
                img->palette[j] = img->palette[k];
                img->palette[k] = t;
                t = img->palette[j+1];
                img->palette[j+1] = img->palette[k+1];
                img->palette[k+1] = t;
                t = img->palette[j+2];
                img->palette[j+2] = img->palette[k+2];
                img->palette[k+2] = t;
            }
        }
    }

}


